"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const boa = require('@pipcook/boa');
const tf = boa.import('tensorflow');
const sys = boa.import('sys');
sys.path.insert(0, path.join(__dirname, '..'));
const { train, evaluate } = boa.import('pyscript.index');
const config = tf.compat.v1.ConfigProto();
config.gpu_options.allow_growth = true;
tf.compat.v1.InteractiveSession(boa.kwargs({
    config: config
}));
/**
 * this is plugin used to train tfjs model with pascal voc data format for image classification problem.
 * @param data : train data
 * @param model : model loaded before
 * @param epochs : need to specify epochs
 * @param batchSize : need to specify batch size
 * @param optimizer : need to specify optimizer
 */
const ModelTrain = (data, model, args) => __awaiter(this, void 0, void 0, function* () {
    try {
        const { epochs = 10, batchSize = 16, modelPath } = args;
        const { trainLoader, validationLoader } = data;
        const count = yield trainLoader.len();
        const valCount = yield validationLoader.len();
        const batchesPerEpoch = Math.floor(count / batchSize);
        const valBatchesPerEpoch = Math.floor(valCount / batchSize);
        const trainModel = model.model;
        for (let i = 0; i < epochs; i++) {
            console.log(`Epoch ${i}/${epochs} start`);
            for (let j = 0; j < batchesPerEpoch; j++) {
                const dataBatch = yield data.trainLoader.nextBatch(batchSize);
                train(dataBatch.map((ele) => ele.data), dataBatch.map((ele) => ele.label), trainModel, j, batchesPerEpoch);
            }
            let loss = 0;
            let accuracy = 0;
            for (let j = 0; j < valBatchesPerEpoch; j++) {
                const dataBatch = yield validationLoader.nextBatch(batchSize);
                const evaluateRes = evaluate(dataBatch.map((ele) => ele.data), dataBatch.map((ele) => ele.label), trainModel);
                // the value returned by evaluate can be either tensor or number
                // if it's tensor, we need to call tensor.numpy() to get numbers
                if (typeof evaluateRes[0] === 'number') {
                    loss += evaluateRes[0];
                }
                else {
                    loss += Number(evaluateRes[0].numpy());
                }
                if (typeof evaluateRes[1] === 'number') {
                    accuracy += evaluateRes[1];
                }
                else {
                    accuracy += Number(evaluateRes[1].numpy());
                }
            }
            loss /= valBatchesPerEpoch;
            accuracy /= valBatchesPerEpoch;
            console.log(`Validation Result ${i}/${epochs} result --- loss: ${loss} accuracy: ${accuracy}`);
        }
        yield fs.ensureDir(modelPath);
        yield trainModel.save_weights(path.join(modelPath, 'weights.h5'));
        yield trainModel.save(path.join(modelPath, 'model.h5'));
        const result = Object.assign({}, model, { model: trainModel });
        return result;
    }
    catch (err) {
        console.error('occurs an error on model trainer', err);
        throw err;
    }
});
exports.default = ModelTrain;
//# sourceMappingURL=index.js.map